@@ -1,16 +1,17 @@
 package Bio::DB::SeqFeature::Store::berkeleydb;
 
-# $Id: berkeleydb.pm 14982 2008-11-11 16:03:45Z lstein $
-
-
 use strict;
 use base 'Bio::DB::SeqFeature::Store';
 use Bio::DB::GFF::Util::Rearrange 'rearrange';
-use Bio::DB::Fasta;
 use DB_File;
-use Fcntl qw(O_RDWR O_CREAT);
+use Fcntl qw(O_RDWR O_CREAT :flock);
+use IO::File;
 use File::Temp 'tempdir';
 use File::Path 'rmtree','mkpath';
+use File::Basename;
+use File::Spec;
+use Carp 'carp','croak';
+
 use constant BINSIZE => 10_000;
 use constant MININT  => -999_999_999_999;
 use constant MAXINT  => 999_999_999_999;
@@ -25,19 +26,19 @@ Bio::DB::SeqFeature::Store::berkeleydb -- Storage and retrieval of sequence anno
 
   # Create a database from the feature files located in /home/fly4.3 and store
   # the database index in the same directory:
-  $db =  Bio::DB::SeqFeature::Store->new( -adaptor => 'berkeleydb',
-                                          -dir     => '/home/fly4.3');
+  my $db = Bio::DB::SeqFeature::Store->new( -adaptor => 'berkeleydb',
+                                            -dir     => '/home/fly4.3');
 
   # Create a database that will monitor the files in /home/fly4.3, but store
   # the indexes in /var/databases/fly4.3
-  $db      = Bio::DB::SeqFeature::Store->new( -adaptor    => 'berkeleydb',
-                                              -dsn        => '/var/databases/fly4.3',
-                                              -dir        => '/home/fly4.3');
+  $db    = Bio::DB::SeqFeature::Store->new( -adaptor => 'berkeleydb',
+                                            -dir     => '/home/fly4.3',
+                                            -dsn     => '/var/databases/fly4.3');
 
   # Create a feature database from scratch
-  $db     = Bio::DB::SeqFeature::Store->new( -adaptor => 'berkeleydb',
-                                             -dsn     => '/var/databases/fly4.3',
-                                             -create  => 1);
+  $db    = Bio::DB::SeqFeature::Store->new( -adaptor => 'berkeleydb',
+                                            -dsn     => '/var/databases/fly4.3',
+                                            -create  => 1);
 
   # get a feature from somewhere
   my $feature = Bio::SeqFeature::Generic->new(...);
@@ -58,7 +59,7 @@ Bio::DB::SeqFeature::Store::berkeleydb -- Storage and retrieval of sequence anno
 
   # use the GFF3 loader to do a bulk write:
   my $loader = Bio::DB::SeqFeature::Store::GFF3Loader->new(-store   => $db,
-                                                           -verbose => 1,
+                                                           -verbose => 0,
                                                            -fast    => 1);
   $loader->load('/home/fly4.3/dmel-all.gff');
 
@@ -140,7 +141,8 @@ new_feature() repeatedly, you can create the database and then bulk
 populate it using the GFF3 loader, or you can monitor a directory of
 preexisting GFF3 and FASTA files and rebuild the indexes whenever one
 or more of the fields changes. The last mode is probably the most
-convenient.
+convenient. Note that the indexer will only pay attention to files
+that end with .gff3, .wig and .fa.
 
 =over 4
 
@@ -182,9 +184,16 @@ Adaptor-specific arguments
  -create            Pass true to create the index files if they don't exist
                     (implies -write=>1)
 
+ -locking           Use advisory locking to avoid one process trying to read
+                    from the database while another is updating it (may not
+                    work properly over NFS).
+
  -temp              Pass true to create temporary index files that will
                     be deleted once the script exits.
 
+ -verbose           Pass true to report autoindexing operations on STDERR.
+                    (default is true).
+
 Examples:
 
 To create an empty database which will be populated using calls to
@@ -219,8 +228,15 @@ As above, but store the source files and index files in separate directories:
                                              -dsn     => '/var/databases/fly4.3',
                                              -dir     => '/home/gff3_files/fly4.3');
 
+To be indexed, files must end with one of .gff3 (GFF3 format), .fa
+(FASTA format) or .wig (WIG format).
+
 B<-autoindex> is an alias for B<-dir>.
 
+You should specify B<-locking> in a multiuser environment, including
+the case in which the database is being used by a web server at the
+same time another user might be updating it.
+
 =back
 
 See L<Bio::DB::SeqFeature::Store> for all the access methods supported
@@ -241,12 +257,19 @@ sub init {
       $is_temporary,
       $write,
       $create,
-     ) = rearrange([['DSN','DB'],
-		   [qw(DIR AUTOINDEX)],
-		   ['TMP','TEMP','TEMPORARY'],
-		   [qw(WRITE WRITABLE)],
-		   'CREATE',
+      $verbose,
+      $locking,
+      ) = rearrange([['DSN','DB'],
+		    [qw(DIR AUTOINDEX)],
+		    ['TMP','TEMP','TEMPORARY'],
+		    [qw(WRITE WRITABLE)],
+		    'CREATE',
+		    'VERBOSE',
+		    [qw(LOCK LOCKING)],
 		  ],@_);
+
+  $verbose = 1 unless defined $verbose;
+
   if ($autoindex) {
     -d $autoindex or $self->throw("Invalid directory $autoindex");
     $directory ||= "$autoindex/indexes";
@@ -258,9 +281,9 @@ sub init {
     $pacname =~ s/:+/_/g;
   }
   $directory = tempdir($pacname.'_XXXXXX',
-		       TMPDIR=>1,
-		       CLEANUP=>1,
-		       DIR=>$directory) if $is_temporary;
+		       TMPDIR  => 1,
+		       CLEANUP => 1,
+		       DIR     => $directory) if $is_temporary;
   mkpath($directory);
   -d $directory or $self->throw("Invalid directory $directory");
 
@@ -273,75 +296,312 @@ sub init {
   $self->default_settings;
   $self->directory($directory);
   $self->temporary($is_temporary);
+  $self->verbose($verbose);
+  $self->locking($locking);
   $self->_delete_databases()    if $create;
+  if ($autoindex && -d $autoindex) {
+      $self->auto_reindex($autoindex);
+  }
+  $self->lock('shared');
+
+  # this step may rebless $self into a subclass
+  # to preserve backward compatibility with older
+  # databases while providing better performance for
+  # new databases.
+  $self->possibly_rebless($create);
+
   $self->_open_databases($write,$create,$autoindex);
   $self->_permissions($write,$create);
   return $self;
 }
 
-sub can_store_parentage { 1 }
+sub version { return 2.0 };
 
-sub post_init {
-  my $self = shift;
-  my ($autodir) = rearrange([['DIR','AUTOINDEX']],@_);
-  return unless $autodir && -d $autodir;
+sub possibly_rebless {
+    my $self   = shift;
+    my $create = shift;
+    my $do_rebless;
 
-  my $maxtime   = 0;
-
-  opendir (my $D,$autodir) or $self->throw("Couldn't open directory $autodir for reading: $!");
-  my @reindex;
-  my $fasta_files_present;
-
-  while (defined (my $node = readdir($D))) {
-    next if $node =~ /^\./;
-    my $path      = "$autodir/$node";
-    next unless -f $path;
-
-    # skip fasta files - the Bio::DB::Fasta module indexes them on its own
-    if ($node =~ /\.(?:fa|fasta|dna)(?:\.gz)?$/) {
-      $fasta_files_present++;
-      next;
+    if ($create) {
+	$do_rebless++;
+    } else {  # probe database
+	my %h;
+	tie (%h,'DB_File',$self->_features_path,O_RDONLY,0666,$DB_HASH) or return;
+	$do_rebless = $h{'.version'} >= 3.0;
     }
 
-    # skip index files
-    next if $node =~ /\.(?:bdb|idx|index|stamp)/;
+    if ($do_rebless) {
+	eval "require Bio::DB::SeqFeature::Store::berkeleydb3";
+	bless $self,'Bio::DB::SeqFeature::Store::berkeleydb3';
+    }
+	
+}
 
-    # skip autosave files, etc
-    next if $node =~ /^\#/;
-    next if $node =~ /~$/;
+sub can_store_parentage { 1 }
 
-    my $mtime = _mtime(\*_);  # not a typo
-    $maxtime   = $mtime if $mtime > $maxtime;
-    push @reindex,$path;
-  }
+sub auto_reindex {
+    my $self    = shift;
+    my $autodir = shift;
+    my $result  = $self->needs_auto_reindexing($autodir);
 
-  close $D;
+    if ($result && %$result) {
+	$self->flag_autoindexing(1);
+	$self->lock('exclusive');
+	$self->reindex_wigfiles($result->{wig},$autodir) if $result->{wig};
+	$self->reindex_ffffiles($result->{fff},$autodir) if $result->{fff};
+	$self->reindex_gfffiles($result->{gff},$autodir) if $result->{gff};
+	$self->dna_db(Bio::DB::Fasta::Subdir->new($autodir));
+	$self->unlock;
+	$self->flag_autoindexing(0);
+    }
 
-  my $timestamp_time  = _mtime($self->_mtime_path) || 0;
+    else {
+	$self->dna_db(Bio::DB::Fasta::Subdir->new($autodir));
+    }
+}
 
-  if ($maxtime > $timestamp_time) {
-    warn "Reindexing... this may take a while.\n";
+sub autoindex_flagfile { 
+    return File::Spec->catfile(shift->directory,'autoindex.pid');
+}
+sub auto_index_in_process {
+    my $self = shift;
+    my $flag_file = $self->autoindex_flagfile;
+    return unless -e $flag_file;
+
+    # if flagfile exists, then check that PID still exists
+    open my $fh,$flag_file or die "Couldn't open $flag_file: $!";
+    my $pid = <$fh>;
+    close $fh;
+    return 1 if kill 0=>$pid;
+    warn "Autoindexing seems to be running in another process, but the process has gone away. Trying to override...";
+    if (unlink $flag_file) {
+	warn "Successfully removed stale PID file." if $self->verbose;
+	warn "Assuming partial reindexing process. Rebuilding indexes from scratch..." if $self->verbose;
+	my $glob = File::Spec->catfile($self->directory,'*');
+	unlink glob($glob);
+	return;
+    } else {
+	croak ("Cannot recover from apparent aborted autoindex process. Please remove files in ",
+	     $self->directory,
+	     " and allow the adaptor to reindex");
+	return 1;
+    }
+}
+
+sub flag_autoindexing {
+    my $self = shift;
+    my $doit = shift;
+    my $flag_file = $self->autoindex_flagfile;
+    if ($doit) {
+	open my $fh,'>',$flag_file or die "Couldn't open $flag_file: $!";
+	print $fh $$;
+	close $fh;
+    } else {
+	unlink $flag_file;
+    }
+}
+
+sub reindex_gfffiles {
+    my $self    = shift;
+    my $files   = shift;
+    my $autodir = shift;
+
+    warn "Reindexing GFF files...\n" if $self->verbose;
+    my $exists = -e $self->_features_path;
+
+    $self->_permissions(1,1);
+    $self->_close_databases();
+    $self->_open_databases(1,!$exists);
+    require Bio::DB::SeqFeature::Store::GFF3Loader
+	unless Bio::DB::SeqFeature::Store::GFF3Loader->can('new');
+    my $loader = Bio::DB::SeqFeature::Store::GFF3Loader->new(-store    => $self,
+							     -sf_class => $self->seqfeature_class,
+							     -verbose  => $self->verbose,
+	) 
+	or $self->throw("Couldn't create GFF3Loader");
+    my %seen;
+    $loader->load(grep {!$seen{$_}++} @$files);
+    $self->_touch_timestamp;
+}
+
+sub reindex_ffffiles {
+    my $self    = shift;
+    my $files   = shift;
+    my $autodir = shift;
+
+    warn "Reindexing FFF files...\n" if $self->verbose;
     $self->_permissions(1,1);
     $self->_close_databases();
     $self->_open_databases(1,1);
-    require Bio::DB::SeqFeature::Store::GFF3Loader
-      unless Bio::DB::SeqFeature::Store::GFF3Loader->can('new');
-    my $loader = Bio::DB::SeqFeature::Store::GFF3Loader->new(-store    => $self,
-							     -sf_class => $self->seqfeature_class) 
-      or $self->throw("Couldn't create GFF3Loader");
-    $loader->load(@reindex);
+    require Bio::DB::SeqFeature::Store::FeatureFileLoader
+	unless Bio::DB::SeqFeature::Store::FeatureFileLoader->can('new');
+    my $loader = Bio::DB::SeqFeature::Store::FeatureFileLoader->new(-store    => $self,
+								    -sf_class => $self->seqfeature_class,
+								    -verbose  => $self->verbose,
+	) 
+	or $self->throw("Couldn't create FeatureFileLoader");
+    my %seen;
+    $loader->load(grep {!$seen{$_}++} @$files);
     $self->_touch_timestamp;
-  }
+}
 
-  if ($fasta_files_present) {
-    my $dna_db = Bio::DB::Fasta->new($autodir);
-    $self->dna_db($dna_db);
-  }
+sub reindex_wigfiles {
+    my $self    = shift;
+    my $files   = shift;
+    my $autodir = shift;
+
+    warn "Reindexing wig files...\n" if $self->verbose;
+
+    unless (Bio::Graphics::Wiggle::Loader->can('new')) {
+	eval "require Bio::Graphics::Wiggle::Loader; 1"
+	    or return;
+    }
+
+    for my $wig (@$files) {
+	warn "Reindexing $wig...\n" if $self->verbose;
+	my ($wib_name) = fileparse($wig,qr/\.[^.]*/);
+	my $gff3_name  = "$wib_name.gff3";
+
+	# unlink all wib files that share the basename
+	my $wib_glob   = File::Spec->catfile($self->directory,"$wib_name*.wib");
+	unlink glob($wib_glob);
+
+	my $loader     = Bio::Graphics::Wiggle::Loader->new($self->directory,$wib_name);
+	my $fh         = IO::File->new($wig) or die "Can't open $wig: $!";
+	$loader->load($fh);  # will create one or more .wib files
+	$fh->close;
+	my $gff3_data  = $loader->featurefile('gff3','microarray_oligo',$wib_name);
+	my $gff3_path  = File::Spec->catfile($autodir,$gff3_name);
+	$fh            = IO::File->new($gff3_path,'>')
+	    or die "Can't open $gff3_path for writing: $!";
+	$fh->print($gff3_data);
+	$fh->close;
+	my $conf_path  = File::Spec->catfile($autodir,"$wib_name.conf");
+	$fh            = IO::File->new($conf_path,'>');
+	$fh->print($loader->conf_stanzas('microarray_oligo',$wib_name));
+	$fh->close;
+    }
+}
+
+# returns the following hashref
+# empty hash if nothing needs reindexing
+# {fasta => 1}                    if DNA database needs reindexing
+# {gff   => [list,of,gff,paths]}  if gff3 files need reindexing
+# {wig   => [list,of,wig,paths]}  if wig files need reindexing
+sub needs_auto_reindexing {
+    my $self    = shift;
+    my $autodir = shift;
+    my $result    = {};
+
+    # don't allow two processes to reindex simultaneously
+	$self->auto_index_in_process and croak "Autoindexing in process. Try again later";
+
+    # first interrogate the GFF3 files, using the timestamp file
+    # as modification comparison
+    my (@gff3,@fff,@wig,$fasta,$fasta_index_time);
+    opendir (my $D,$autodir) 
+	or $self->throw("Couldn't open directory $autodir for reading: $!");
+
+    my $maxtime   = 0;
+    my $timestamp_time  = _mtime($self->_mtime_path) || 0;
+    while (defined (my $node = readdir($D))) {
+	next if $node =~ /^\./;
+	my $path      = File::Spec->catfile($autodir,$node);
+	next unless -f $path;
+
+	if ($path =~ /\.gff\d?$/i) {
+	    my $mtime = _mtime(\*_);  # not a typo
+	    $maxtime   = $mtime if $mtime > $maxtime;
+	    push @gff3,$path;
+	}
+	
+	
+	elsif ($path =~ /\.fff?$/i) {
+	    my $mtime = _mtime(\*_);  # not a typo
+	    $maxtime   = $mtime if $mtime > $maxtime;
+	    push @fff,$path;
+	}
+	
+	elsif ($path =~ /\.wig$/i) {
+	    my $wig       = $path;
+	    (my $gff_file = $wig) =~ s/\.wig$/\.gff3/i;
+	    next if -e $gff_file && _mtime($gff_file) > _mtime($path);
+	    push @wig,$wig;
+	    push @gff3,$gff_file;
+	    $maxtime      = time();
+	}
+
+	elsif ($path =~ /\.(fa|fasta|dna)$/i) {
+	    $fasta_index_time = 
+		_mtime(File::Spec->catfile($self->directory,'fasta.index'))||0
+		unless defined $fasta_index_time;
+	    $fasta++ if _mtime($path) > $fasta_index_time;
+	}
+    }
+    closedir $D;
+    
+    $result->{gff}     = \@gff3 if $maxtime > $timestamp_time;
+    $result->{wig}     = \@wig  if @wig;
+    $result->{fff}     = \@fff  if @fff;
+    $result->{fasta}++ if $fasta;
+    return $result;
+}
+
+sub verbose {
+    my $self = shift;
+    my $d    = $self->{verbose};
+    $self->{verbose} = shift if @_;
+    return $d;
+}
+
+sub locking {
+    my $self = shift;
+    my $d    = $self->{locking};
+    $self->{locking} = shift if @_;
+    return $d;
+}
+
+sub lockfile {
+    my $self = shift;
+    return File::Spec->catfile($self->directory,'lock');
+}
+
+sub lock {
+    my $self = shift;
+    my $mode = shift;
+    return unless $self->locking;
+
+    my $flag = $mode eq 'exclusive' ? LOCK_EX : LOCK_SH;
+    my $lockfile = $self->lockfile;
+    my $fh = $self->_flock_fh;
+    unless ($fh) {
+	my $open  = -e $lockfile ? '<' : '>';
+	$fh       = IO::File->new($lockfile,$open) or die "Cannot open $lockfile: $!";
+    }
+    flock($fh,$flag);
+    $self->_flock_fh($fh);
+}
+
+sub unlock {
+    my $self = shift;
+    return unless $self->locking;
+
+    my $fh = $self->_flock_fh or return;
+    flock($fh,LOCK_UN);
+    undef $self->{flock_fh};
+}
+
+sub _flock_fh {
+    my $self = shift;
+    my $d    = $self->{flock_fh};
+    $self->{flock_fh} = shift if @_;
+    $d;
 }
 
 sub _open_databases {
   my $self = shift;
   my ($write,$create,$ignore_errors) = @_;
+  return if $self->db; # already open - don't reopen
 
   my $directory  = $self->directory;
   unless (-d $directory) {  # directory does not exist
@@ -356,55 +616,84 @@ sub _open_databases {
   # Create the main database; this is a DB_HASH implementation
   my %h;
   my $result = tie (%h,'DB_File',$self->_features_path,$flags,0666,$DB_HASH);
+
   unless ($result) {
     return if $ignore_errors;  # autoindex set, so defer this
     $self->throw("Couldn't tie: ".$self->_features_path . " $!");
   }
+
   if ($create) {
     %h = ();
     $h{'.next_id'} = 1;
+    $h{'.version'} = $self->version;
   }
   $self->db(\%h);
 
-  # Create the index databases; these are DB_BTREE implementations with duplicates allowed.
-  local $DB_BTREE->{flags} = R_DUP;
-  $DB_BTREE->{compare}     = sub { lc($_[0]) cmp lc($_[1]) };
-  for my $idx ($self->_index_files) {
-    my $path = $self->_qualify("$idx.idx");
-    my %db;
-    tie(%db,'DB_File',$path,$flags,0666,$DB_BTREE)
-      or $self->throw("Couldn't tie $path: $!");
-    %db = () if $create;
-    $self->index_db($idx=>\%db);
-  }
+  $self->open_index_dbs($flags,$create);
+  $self->open_parentage_db($flags,$create);
+  $self->open_notes_db($write,$create);
+  $self->open_seq_db($flags,$create) if -e $self->_fasta_path;
+}
 
-  # Create the parentage database
-  my %p;
-  tie (%p,'DB_File',$self->_parentage_path,$flags,0666,$DB_BTREE)
-    or $self->throw("Couldn't tie: ".$self->_parentage_path . $!);
+sub open_index_dbs {
+    my $self = shift;
+    my ($flags,$create) = @_;
+
+    # Create the index databases; these are DB_BTREE implementations with duplicates allowed.
+    $DB_BTREE->{flags}  = R_DUP;
+    $DB_BTREE->{compare}     = sub { lc($_[0]) cmp lc($_[1]) };
+    for my $idx ($self->_index_files) {
+	my $path = $self->_qualify("$idx.idx");
+	my %db;
+	my $result = tie(%db,'DB_File',$path,$flags,0666,$DB_BTREE);
+	# for backward compatibility, allow a failure when trying to open the is_indexed index.
+	$self->throw("Couldn't tie $path: $!") unless $result || $idx eq 'is_indexed';
+	%db = () if $create;
+	$self->index_db($idx=>\%db);
+    }
+}
+
+sub open_parentage_db {
+    my $self = shift;
+    my ($flags,$create) = @_;
+
+    # Create the parentage database
+    my %p;
+    tie (%p,'DB_File',$self->_parentage_path,$flags,0666,$DB_BTREE)
+	or $self->throw("Couldn't tie: ".$self->_parentage_path . $!);
     %p = () if $create;
-  $self->parentage_db(\%p);
+    $self->parentage_db(\%p);
+}
 
-  if (-e $self->_fasta_path) {
-    my $dna_db = Bio::DB::Fasta->new($self->_fasta_path) or $self->throw("Can't reindex sequence file: $@");
-    $self->dna_db($dna_db);
-  }
+sub open_notes_db {
+    my $self = shift;
+    my ($write,$create) = @_;
+    
+    my $mode =  $write  ? "+>>"
+	                : $create ? "+>"
+	                : "<";
 
-  my $mode =  $write  ? "+>>"
-            : $create ? "+>"
-            : "<";
+    open (my $F,$mode,$self->_notes_path) or $self->throw($self->_notes_path.": $!");
+    $self->notes_db($F);
+}
 
-  open (my $F,$mode,$self->_notes_path) or $self->throw($self->_notes_path.": $!");
-  $self->notes_db($F);
+sub open_seq_db {
+    my $self = shift;
+
+    if (-e $self->_fasta_path) {
+	my $dna_db = Bio::DB::Fasta::Subdir->new($self->_fasta_path) 
+	    or $self->throw("Can't reindex sequence file: $@");
+	$self->dna_db($dna_db);
+    }
 }
 
 sub commit { # reindex fasta files
   my $self = shift;
   if (my $fh = $self->{fasta_fh}) {
     $fh->close;
-    $self->dna_db(Bio::DB::Fasta->new($self->{fasta_file}));
+    $self->dna_db(Bio::DB::Fasta::Subdir->new($self->{fasta_file}));
   } elsif (-d $self->directory) {
-    $self->dna_db(Bio::DB::Fasta->new($self->directory));
+    $self->dna_db(Bio::DB::Fasta::Subdir->new($self->directory));
   }
 }
 
@@ -413,6 +702,7 @@ sub _close_databases {
   $self->db(undef);
   $self->dna_db(undef);
   $self->notes_db(undef);
+  $self->parentage_db(undef);
   $self->index_db($_=>undef) foreach $self->_index_files;
 }
 
@@ -443,12 +733,14 @@ sub _store {
   my $self    = shift;
   my $indexed = shift;
   my $db   = $self->db;
+  my $is_indexed = $self->index_db('is_indexed');
   my $count = 0;
   for my $obj (@_) {
     my $primary_id = $obj->primary_id;
     $self->_delete_indexes($obj,$primary_id)  if $indexed && $primary_id;
     $primary_id    = $db->{'.next_id'}++      unless defined $primary_id;
     $db->{$primary_id} = $self->freeze($obj);
+    $is_indexed->{$primary_id} = $indexed if $is_indexed;
     $obj->primary_id($primary_id);
     $self->_update_indexes($obj)              if $indexed;
     $count++;
@@ -459,7 +751,6 @@ sub _store {
 sub _delete_indexes {
   my $self = shift;
   my ($obj,$id) = @_;
-  warn $obj->display_name;
 
   # the additional "1" causes the index to be deleted
   $self->_update_name_index($obj,$id,1);
@@ -487,8 +778,9 @@ sub _add_SeqFeature {
   for my $child (@children) {
     my $child_id = ref $child ? $child->primary_id : $child;
     defined $child_id or $self->throw("no primary ID known for $child");
-    $p->{$parent_id} = $child_id;
+    $p->{$parent_id} = $child_id if tied(%$p)->find_dup($parent_id,$child_id);
   }
+  return scalar @children;
 }
 
 sub _fetch_SeqFeatures {
@@ -503,10 +795,19 @@ sub _fetch_SeqFeatures {
   my @children      = map {$self->fetch($_)} @children_ids;
 
   if (@types) {
-    my $regexp = join '|',map {quotemeta($_)} $self->find_types(@types);
-    return grep {($_->primary_tag.':'.$_->source_tag) =~ /^$regexp$/i} @children;
+      foreach (@types) { 
+	  my ($a,$b) = split ':',$_,2;
+	  $_  = quotemeta($a);
+	  if (length $b) {
+	      $_ .= ":".quotemeta($b).'$';
+	  } else {
+	      $_ .= ':';
+	  }
+      }
+      my $regexp = join '|', @types;
+      return grep {($_->primary_tag.':'.$_->source_tag) =~ /^($regexp)/i} @children;
   } else {
-    return @children;
+      return @children;
   }
 }
 
@@ -572,7 +873,7 @@ sub _update_location_index {
   my $bin_max     = int $end/BINSIZE;
 
   for (my $bin = $bin_min; $bin <= $bin_max; $bin++ ) {
-    my $key = sprintf("%s%06d",lc($seq_id),$bin);
+    my $key = sprintf("%s.%06d",lc($seq_id),$bin);
     $self->update_or_delete($delete,$db,$key,pack("i4",$id,$start,$end,$strand));
   }
 }
@@ -646,6 +947,14 @@ sub notes_db {
   $d;
 }
 
+# the is_indexed_db 
+sub is_indexed_db {
+  my $self = shift;
+  my $d = $self->setting('is_indexed_db');
+  $self->setting(is_indexed_db=>shift) if @_;
+  $d;
+}
+
 # The indicated index berkeley db
 sub index_db {
   my $self = shift;
@@ -664,7 +973,7 @@ sub _mtime {
 
 # return names of all the indexes
 sub _index_files {
-  return qw(names types locations attributes);
+  return qw(names types locations attributes is_indexed);
 }
 
 # the directory in which we store our indexes
@@ -757,7 +1066,9 @@ sub _features {
 
   my @result;
   unless (defined $name or defined $seq_id or defined $types or defined $attributes) {
-    @result = grep {$_ ne '.next_id' } keys %{$self->db};
+      my $is_indexed = $self->index_db('is_indexed');
+      @result = $is_indexed ? grep {$is_indexed->{$_}} keys %{$self->db}
+                            : grep { !/^\./ }keys %{$self->db};
   }
 
   my %found = ();
@@ -805,7 +1116,8 @@ sub filter_by_name {
   for (my $status = $db->seq($key,$value,R_CURSOR);
        $status == 0 and $key =~ /^$regexp$/i;
        $status = $db->seq($key,$value,R_NEXT)) {
-    push @results,$value;
+      next if %$filter && !$filter->{$value};  # don't bother
+      push @results,$value;
   }
   $self->update_filter($filter,\@results);
 }
@@ -833,12 +1145,24 @@ sub filter_by_type {
     my $key   = lc "$primary_tag:$source_tag";
     my $value;
 
-    for (my $status = $db->seq($key,$value,R_CURSOR);
- 	 $status == 0 && $key =~ /$match/i;
-	 $status = $db->seq($key,$value,R_NEXT)) {
-      push @results,$value;
+    # If filter is already provided, then it is usually faster to
+    # fetch each object.
+    if (%$filter) {  
+	for my $id (keys %$filter) {
+	    my $obj = $self->_fetch($id) or next;
+	    push @results,$id if $obj->type =~ /$match/i;
+	}
+
     }
 
+    else {
+	for (my $status = $db->seq($key,$value,R_CURSOR);
+	     $status == 0 && $key =~ /$match/i;
+	     $status = $db->seq($key,$value,R_NEXT)) {
+	    next if %$filter && !$filter->{$value};  # don't even bother
+	    push @results,$value;
+	}
+    }
   }
   $self->update_filter($filter,\@results);
 }
@@ -859,11 +1183,13 @@ sub filter_by_location {
 
   $start = MININT  if !defined $start;
   $end   = MAXINT  if !defined $end;
+  my $version_2 = $self->db_version > 1;
 
   if ($range_type eq 'overlaps' or $range_type eq 'contains') {
-    my $key     = "\L$seq_id\E$binstart";
-    my $keystop = "\L$seq_id\E$binend";
+    my $key     = $version_2 ? "\L$seq_id\E.$binstart" : "\L$seq_id\E$binstart";
+    my $keystop = $version_2 ? "\L$seq_id\E.$binend"   : "\L$seq_id\E$binend";
     my $value;
+
     for (my $status = $db->seq($key,$value,R_CURSOR);
 	 $status == 0 && $key le $keystop;
 	 $status = $db->seq($key,$value,R_NEXT)) {
@@ -876,6 +1202,7 @@ sub filter_by_location {
       elsif ($range_type eq 'contains') {
 	next unless $fstart >= $start && $fend <= $end;
       }
+      next if %$filter && !$filter->{$id};  # don't bother
       push @results,$id;
     }
   }
@@ -883,8 +1210,8 @@ sub filter_by_location {
   # for contained in, we look for features originating and terminating outside the specified range
   # this is incredibly inefficient, but fortunately the query is rare (?)
   elsif ($range_type eq 'contained_in') {
-    my $key     = "\L$seq_id";
-    my $keystop = "\L$seq_id\E$binstart";
+    my $key     = $version_2 ? "\L$seq_id."            : "\L$seq_id";
+    my $keystop = $version_2 ? "\L$seq_id\E.$binstart" : "\L$seq_id\E$binstart";
     my $value;
 
     # do the left part of the range
@@ -895,11 +1222,12 @@ sub filter_by_location {
       next if $seenit{$id}++;
       next if $strand && $fstrand != $strand;
       next unless $fstart <= $start && $fend >= $end;
+      next if %$filter && !$filter->{$id};  # don't bother
       push @results,$id;
     }
 
     # do the right part of the range
-    $key = "\L$seq_id\E$binend";
+    $key = "\L$seq_id\E.$binend";
     for (my $status = $db->seq($key,$value,R_CURSOR);
 	 $status == 0;
 	 $status = $db->seq($key,$value,R_NEXT)) {
@@ -907,6 +1235,7 @@ sub filter_by_location {
       next if $seenit{$id}++;
       next if $strand && $fstrand != $strand;
       next unless $fstart <= $start && $fend >= $end;
+      next if %$filter && !$filter->{$id};  # don't bother
       push @results,$id;
     }
 
@@ -944,7 +1273,8 @@ sub filter_by_attribute {
       for (my $status = $db->seq($key,$value,R_CURSOR);
 	   $status == 0 && $key =~ /^$att_name:$regexp$/i;
 	   $status = $db->seq($key,$value,R_NEXT)) {
-	push @result,$value;
+	  next if %$filter && !$filter->{$value};  # don't bother
+	  push @result,$value;
       }
     }
     $result ||= $self->update_filter($filter,\@result);
@@ -1107,15 +1437,22 @@ sub finish_bulk_update {
   my $self = shift;
   if (my $fh = $self->{fasta_fh}) {
     $fh->close;
-    $self->{fasta_db} = Bio::DB::Fasta->new($self->{fasta_file});
+    $self->{fasta_db} = Bio::DB::Fasta::Subdir->new($self->{fasta_file});
   }
 }
 
+sub db_version {
+    my $self = shift;
+    my $db   = $self->db;
+    return $db->{'.version'} || 1.00;
+}
+
 
 sub DESTROY {
   my $self = shift;
   $self->_close_databases();
-  rmtree($self->directory,0,1) if $self->temporary;
+  $self->private_fasta_file->close;
+  rmtree($self->directory,0,1) if $self->temporary && -e $self->directory;
 }
 
 # TIE interface -- a little annoying because we are storing magic ".variable"
@@ -1154,6 +1491,7 @@ sub _deleteid {
   my $obj  = $self->fetch($id) or return;
   $self->_delete_indexes($obj,$id);
   delete $self->db->{$id};
+  1;
 }
 
 sub _clearall {
@@ -1188,6 +1526,29 @@ sub next_seq {
   return $store->fetch($id);
 }
 
+
+package Bio::DB::Fasta::Subdir;
+
+use base 'Bio::DB::Fasta';
+
+# alter calling arguments so that the index file is placed in a subdirectory
+# named "indexes"
+
+sub new {
+    my ($class, $path, %opts) = @_;
+    if (-d $path) {
+        $opts{-index_name} = File::Spec->catfile($path,'indexes','fasta.index');
+    }
+    return Bio::DB::Fasta->new($path, %opts);
+}
+
+
+sub _calculate_offsets {
+    my ($self, @args) = @_;
+    return $self->SUPER::_calculate_offsets(@args);
+}
+
+
 1;
 
 __END__
@@ -1217,4 +1578,3 @@ This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
 =cut
-
