@@ -1,4 +1,3 @@
-# $Id: SiteMatrix.pm 11480 2007-06-14 14:16:21Z sendu $
 #---------------------------------------------------------
 
 =head1 NAME
@@ -121,13 +120,24 @@ of the Bioperl mailing lists.  Your participation is much appreciated.
   bioperl-l@bioperl.org                  - General discussion
   http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
 
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
+
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution.  Bug reports can be submitted via the
 web:
 
-  http://bugzilla.open-bio.org/
+  https://redmine.open-bio.org/projects/bioperl/
 
 =head1 AUTHOR - Stefan Kirov
 
@@ -332,7 +342,7 @@ sub calc_weight {
     $model{probC}=$model->{C};
     $model{probG}=$model->{G};
     $model{probT}=$model->{T};
-    foreach my $let qw(probA probC probG probT) {
+    foreach my $let (qw(probA probC probG probT)) {
       my @str;
       $self->throw('You did not provide valid model\n') unless (($model{$let}>0) && ($model{$let}<1));
       foreach my $f (@{$self->{$let}}) {
