@@ -1,7 +1,8 @@
-# $Id: GenericHSP.pm 15084 2008-12-03 22:31:23Z cjfields $
 #
 # BioPerl module for Bio::Search::HSP::GenericHSP
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Jason Stajich <jason@bioperl.org>
 #
 # Copyright Jason Stajich
@@ -75,13 +76,24 @@ the Bioperl mailing list.  Your participation is much appreciated.
   bioperl-l@bioperl.org                  - General discussion
   http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
 
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
+
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
 of the bugs and their resolution. Bug reports can be submitted via the
 web:
 
-  http://bugzilla.open-bio.org/
+  https://redmine.open-bio.org/projects/bioperl/
 
 =head1 AUTHOR - Jason Stajich and Steve Chervitz
 
@@ -203,17 +215,13 @@ sub new {
 
 sub _logical_length {
     my ($self, $type) = @_;
-    my $algo = $self->algorithm;
-    my $len = $self->length($type);
-    my $logical = $len;
-    if($algo =~ /^(PSI)?T(BLAST|FAST)[NY]/oi ) {
-        $logical = $len/3 if $type =~ /sbjct|hit|tot/i;
-    } elsif($algo =~ /^(BLAST|FAST)(X|Y|XY)/oi ) {
-        $logical = $len/3 if $type =~ /query|tot/i;
-    } elsif($algo =~ /^T(BLAST|FAST)(X|Y|XY)/oi ) {
-        $logical = $len/3;
-    }
-    return $logical;
+    if (!defined($self->{_sbjct_offset}) || !defined($self->{_query_offset})) {
+        $self->_calculate_seq_offsets();
+    }    
+    my $key = $type =~ /sbjct|hit|tot/i ? 'sbjct' : 'query';
+    
+    my $offset = $self->{"_${key}_offset"};
+    return $self->length($type) / $offset ;
 }
 
 =head2 L<Bio::Search::HSP::HSPI> methods
@@ -293,6 +301,7 @@ sub evalue {
                              synonyms: 'hsp'
                    default = 'total'
            arg 2: [optional] frac identical value to set for the type requested
+ Note    : for translated sequences, this adjusts the length accordingly
 
 =cut
 
@@ -614,6 +623,7 @@ sub get_aln {
     my ($self) = @_;
     require Bio::LocatableSeq;
     require Bio::SimpleAlign;
+
     my $aln = Bio::SimpleAlign->new();
     my $hs = $self->hit_string();
     my $qs = $self->query_string();
@@ -633,22 +643,26 @@ sub get_aln {
     #}
     
     # mapping: 1 residues for every x coordinate positions
-    my $query = Bio::LocatableSeq->new('-seq'   => $qs,
-                                      '-id'    => $q_nm,
-                                      '-start' => $self->query->start,
-                                      '-end'   => $self->query->end,
-                                      '-force_nse' => $q_nm ? 0 : 1,
-                                      '-mapping' => [1, $self->{_query_mapping}]
-                                      );
+    my $query = Bio::LocatableSeq->new(
+        -seq       => $qs,
+        -id        => $q_nm,
+        -start     => $self->query->start,
+        -end       => $self->query->end,
+        -strand    => $self->query->strand,
+        -force_nse => $q_nm ? 0 : 1,
+        -mapping   => [ 1, $self->{_query_mapping} ]
+    );
     $seqonly = $hs;
     $seqonly =~ s/[\-\s]//g;
-    my $hit =  Bio::LocatableSeq->new('-seq'    => $hs,
-                                      '-id'    => $s_nm,
-                                      '-start' => $self->hit->start,
-                                      '-end'   => $self->hit->end,
-                                      '-force_nse' => $s_nm ? 0 : 1,
-                                      '-mapping' => [1, $self->{_hit_mapping}]
-                                      );
+    my $hit = Bio::LocatableSeq->new(
+        -seq       => $hs,
+        -id        => $s_nm,
+        -start     => $self->hit->start,
+        -end       => $self->hit->end,
+        -strand    => $self->hit->strand,
+        -force_nse => $s_nm ? 0 : 1,
+        -mapping   => [ 1, $self->{_hit_mapping} ]
+    );
     $aln->add_seq($query);
     $aln->add_seq($hit);
     return $aln;
@@ -742,7 +756,7 @@ sub rank {
            :             'conserved-not-identical' - conserved positions w/o 
            :                            identical residues
            :             The name can be shortened to 'id' or 'cons' unless
-           :             the name is ambiguous.  The default value is
+           :             the name is .  The default value is
            :             'identical'
            :
            : collapse  = boolean, if true, consecutive positions are merged
@@ -861,9 +875,12 @@ See Also   : L<Bio::Search::Hit::HSPI::seq_inds()>
 
 sub ambiguous_seq_inds {
     my $self = shift;
-    $self->_calculate_seq_positions();    
-    return $self->{seqinds}{'_warnRes'} if exists $self->{seqinds}{'_warnRes'};
-    return $self->{seqinds}{'_warnRes'} = '';
+    $self->_calculate_seq_positions();
+    my $type = ($self->{_query_offset} == 3 && $self->{_sbjct_offset} == 3) ?
+        'query/subject' :
+        ($self->{_query_offset} == 3) ? 'query' :
+        ($self->{_sbjct_offset} == 3) ? 'subject' : '';
+    return $type;
 }
 
 =head2 Inherited from L<Bio::SeqFeature::SimilarityPair>
@@ -947,13 +964,39 @@ sub feature2 {
 # not defined (WU-BLAST), return the p-value.
 sub significance {
     my ($self, $val) = @_;
-    if (!defined $self->{SIGNIFICANCE} || $val) {
-        $self->query->significance($val) if $val;
-        $self->{SIGNIFICANCE} = $val || $self->evalue || $self->pvalue;
+    if (!defined $self->{SIGNIFICANCE} || defined $val) {
+        $self->{SIGNIFICANCE} = defined $val ? $val :
+                                defined $self->evalue ?  $self->evalue :
+                                defined $self->pvalue ? $$self->pvalue :
+                                undef;        
+        $self->query->significance($self->{SIGNIFICANCE});
     }
     return $self->{SIGNIFICANCE};
 }
 
+=head2 strand
+
+ Title   : strand
+ Usage   : $hsp->strand('query')
+ Function: Retrieves the strand for the HSP component requested
+ Returns : +1 or -1
+ Args    : 'hit' or 'subject' or 'sbjct' to retrieve the strand of the subject,
+           'query' to retrieve the query strand (default)
+
+=cut
+
+sub strand {
+    my ($self,$type) = @_;
+
+    if( $type =~ /^q/i && defined $self->{'QUERY_STRAND'} ) {
+        return $self->{'QUERY_STRAND'};
+    } elsif( $type =~ /^(hit|subject|sbjct)/i && defined $self->{'HIT_STRAND'} ) {
+        return $self->{'HIT_STRAND'};
+    } 
+
+    return $self->SUPER::strand($type)
+}
+
 =head2 score
 
  Title   : score
@@ -963,7 +1006,6 @@ sub significance {
  Returns : numeric
  Args    : [optional] new value to set
 
-
 =head2 bits
 
  Title   : bits
@@ -973,18 +1015,6 @@ sub significance {
  Returns : numeric
  Args    : [optional] new value to set
 
-
-=head2 strand
-
- Title   : strand
- Usage   : $hsp->strand('query')
- Function: Retrieves the strand for the HSP component requested
- Returns : +1 or -1 (0 if unknown)
- Args    : 'hit' or 'subject' or 'sbjct' to retrieve the strand of the subject
-           'query' to retrieve the query strand (default)
-
-=cut
-
 =head1 Private methods
 
 =cut
@@ -1052,20 +1082,10 @@ sub _calculate_seq_positions {
         #$sseq =~ s![\\\/]!!g;
     }
 
-    $self->{seqinds}{'_warnRes'} = '';
-    ($self->{_sbjct_offset}, $self->{_query_offset}) = (1,1);
-    if($prog =~ /^(?:PSI)?T(BLAST|FAST)(N|X|Y)/oi ) {
-        $self->{_sbjct_offset} = 3;
-        if ($1 eq 'BLAST' && $2 eq 'X') { #TBLASTX
-            $self->{_query_offset} = 3;
-            $self->{seqinds}{'_warnRes'} = 'query/subject';
-        } else {
-            $self->{seqinds}{'_warnRes'} = 'subject';
-        }
-    } elsif($prog =~ /^(BLAST|FAST)(X|Y|XY)/oi  ) {
-        $self->{_query_offset} = 3;
-        $self->{seqinds}{'_warnRes'} = 'query';
+    if (!defined($self->{_sbjct_offset}) || !defined($self->{_query_offset})) {
+        $self->_calculate_seq_offsets();
     }
+    
     my ($qfs, $sfs) = (0,0);
     CHAR_LOOP:
     for my $pos (0..CORE::length($seqString)-1) {
@@ -1166,6 +1186,21 @@ sub _calculate_seq_positions {
     return 1;
 }
 
+sub _calculate_seq_offsets {
+    my $self = shift;
+    my $prog = $self->algorithm;
+    ($self->{_sbjct_offset}, $self->{_query_offset}) = (1,1);
+    if($prog =~ /^(?:PSI)?T(BLAST|FAST)(N|X|Y)/oi ) {
+        $self->{_sbjct_offset} = 3;
+        if ($1 eq 'BLAST' && $2 eq 'X') { #TBLASTX
+            $self->{_query_offset} = 3;
+        } 
+    } elsif($prog =~ /^(BLAST|FAST)(X|Y|XY)/oi  ) {
+        $self->{_query_offset} = 3;
+    }
+    1;
+}
+
 =head2 n
 
 See documentation in L<Bio::Search::HSP::HSPI::n()|Bio::Search::HSP::HSPI>
@@ -1175,7 +1210,8 @@ See documentation in L<Bio::Search::HSP::HSPI::n()|Bio::Search::HSP::HSPI>
 sub n {
     my $self = shift;
     if(@_) { $self->{'N'} = shift; }
-    defined $self->{'N'} ? $self->{'N'} : '';
+    # note that returning 1 is completely an assumption
+    defined $self->{'N'} ? $self->{'N'} : 1;
 }
 
 =head2 range
@@ -1396,7 +1432,7 @@ sub _pre_seq_feature {
         $queryfactor = 1;
         $querymap = 3;
     }
-    elsif ($algo =~ /^T(BLAST|FAST|SW)(X|Y|XY)/oi || $algo =~ /^(BLAST|FAST|SW)N/oi || $algo =~ /^WABA|AXT|BLAT|BLASTZ|PSL|MEGABLAST|EXONERATE|SW|SMITH\-WATERMAN|SIM4$/){
+    elsif ($algo =~ /^T(BLAST|FAST|SW)(X|Y|XY)/oi || $algo =~ /^(BLAST|FAST|SW)N/oi || $algo =~ /^WABA|AXT|BLAT|BLASTZ|PSL|MEGABLAST|EXONERATE|SW|SSEARCH|SMITH\-WATERMAN|SIM4$/){
         if ($2) {
             $hitmap = $querymap = 3;
         }
@@ -1601,6 +1637,7 @@ sub _pre_similar_stats {
 
 sub _pre_frac {
     my $self = shift;
+    
     my $hsp_len = $self->{HSP_LENGTH};
     my $hit_len = $self->{HIT_LENGTH};
     my $query_len = $self->{QUERY_LENGTH};
