@@ -1,7 +1,8 @@
-# $Id: Seq.pm 11480 2007-06-14 14:16:21Z sendu $
 #
 # BioPerl module for Bio::Seq
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Ewan Birney <birney@ebi.ac.uk>
 #
 # Copyright Ewan Birney
@@ -31,7 +32,7 @@ Bio::Seq - Sequence object, with features
 
     # make from strings in script
     $seqobj = Bio::Seq->new( -display_id => 'my_id',
-			     -seq => $sequence_as_string);
+                             -seq => $sequence_as_string);
 
     # gets sequence as a string from sequence object
     $seqstr   = $seqobj->seq(); # actual sequence as a string
@@ -42,8 +43,8 @@ Bio::Seq - Sequence object, with features
 
     @features = $seqobj->get_SeqFeatures(); # just top level
     foreach my $feat ( @features ) {
-	print "Feature ",$feat->primary_tag," starts ",$feat->start," ends ",
-	$feat->end," strand ",$feat->strand,"\n";
+        print "Feature ",$feat->primary_tag," starts ",$feat->start," ends ",
+            $feat->end," strand ",$feat->strand,"\n";
 
         # features retain link to underlying sequence object
         print "Feature sequence is ",$feat->seq->seq(),"\n"
@@ -52,7 +53,7 @@ Bio::Seq - Sequence object, with features
     # sequences may have a species
 
     if( defined $seq->species ) {
-	print "Sequence is from ",$species->binomial," [",$species->common_name,"]\n";
+        print "Sequence is from ",$species->binomial," [",$species->common_name,"]\n";
     }
 
     # annotation objects are Bio::AnnotationCollectionI's
@@ -63,7 +64,7 @@ Bio::Seq - Sequence object, with features
     # more information
 
     foreach my $ref ( $ann->get_Annotations('reference') ) {
-	print "Reference ",$ref->title,"\n";
+        print "Reference ",$ref->title,"\n";
     }
 
     # you can get truncations, translations and reverse complements, these
@@ -135,8 +136,8 @@ exons it finds as sequence features in Genbank Flat File format.
       print "Seen sequence ",$seqobj->display_id,", start of seq ",
             substr($seqobj->seq,1,10),"\n";
       if( $seqobj->alphabet eq 'dna') {
-	    $rev = $seqobj->revcom;
-	    $id  = $seqobj->display_id();
+            $rev = $seqobj->revcom;
+            $id  = $seqobj->display_id();
             $id  = "$id.rev";
             $rev->display_id($id);
             $seqout->write_seq($rev);
@@ -146,7 +147,7 @@ exons it finds as sequence features in Genbank Flat File format.
            if( $feat->primary_tag eq 'exon' ) {
               print STDOUT "Location ",$feat->start,":",
                     $feat->end," GFF[",$feat->gff_string,"]\n";
-	   }
+           }
       }
   }
 
@@ -255,7 +256,7 @@ more information).
   $seqobj->subseq(5,10);       # part of the sequence as a string
   $seqobj->accession_number(); # when there, the accession number
   $seqobj->alphabet();         # one of 'dna','rna',or 'protein'
-  $seqobj->seq_version()       # when there, the version
+  $seqobj->version()           # when there, the version
   $seqobj->keywords();         # when there, the Keywords line
   $seqobj->length()            # length
   $seqobj->desc();             # description
@@ -352,7 +353,6 @@ See L<Bio::PrimarySeq> for more details on PrimarySeq objects.
   # one can also add features
 
   $seqobj->add_SeqFeature($feat)     # returns 1 if successful
-  $seqobj->add_SeqFeature(@features) # returns 1 if successful
 
   # sub features. For complex join() statements, the feature
   # is one sequence feature with many sub SeqFeatures
@@ -374,7 +374,7 @@ in order to retrieve the coordinates inside the Genbank or EMBL join()
 statements (e.g. "CDS    join(51..142,273..495,1346..1474)"):
 
   if ( $feat->location->isa('Bio::Location::SplitLocationI') &&
-	       $feat->primary_tag eq 'CDS' )  {
+               $feat->primary_tag eq 'CDS' )  {
     foreach $loc ( $feat->location->sub_Location ) {
       print $loc->start . ".." . $loc->end . "\n";
     }
@@ -415,13 +415,24 @@ of the Bioperl mailing lists. Your participation is much appreciated.
   bioperl-l@bioperl.org                  - General discussion
   http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
 
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
+
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution.  Bug reports can be submitted via the
 web:
 
-  http://bugzilla.open-bio.org/
+  https://redmine.open-bio.org/projects/bioperl/
 
 =head1 AUTHOR - Ewan Birney, inspired by Ian Korf objects
 
@@ -429,7 +440,8 @@ Email birney@ebi.ac.uk
 
 =head1 CONTRIBUTORS
 
-Jason Stajich E<lt>jason@bioperl.orgE<gt>
+ Jason Stajich E<lt>jason@bioperl.orgE<gt>
+ Mark A. Jensen maj -at- fortinbras -dot- us
 
 =head1 APPENDIX
 
@@ -449,15 +461,15 @@ use strict;
 use Bio::Annotation::Collection;
 use Bio::PrimarySeq;
 
-use base qw(Bio::Root::Root Bio::SeqI Bio::IdentifiableI Bio::DescribableI Bio::AnnotatableI Bio::FeatureHolderI);
+use base qw(Bio::Root::Root Bio::SeqI Bio::IdentifiableI Bio::DescribableI Bio::AnnotatableI Bio::FeatureHolderI Bio::AnnotationCollectionI);
 
 =head2 new
 
  Title   : new
  Usage   : $seq = Bio::Seq->new( -seq => 'ATGGGGGTGGTGGTACCCT',
                                  -id  => 'human_id',
-				 -accession_number => 'AL000012',
-			       );
+                                 -accession_number => 'AL000012',
+                               );
 
  Function: Returns a new Seq object from
            basic constructors, being a string for the sequence
@@ -470,10 +482,10 @@ sub new {
     my($caller,@args) = @_;
 
     if( $caller ne 'Bio::Seq') {
-	$caller = ref($caller) if ref($caller);
+        $caller = ref($caller) if ref($caller);
     }
 
-    # we know our inherietance heirarchy
+    # we know our inherietance hierarchy
     my $self = Bio::Root::Root->new(@args);
     bless $self,$caller;
 
@@ -499,28 +511,29 @@ sub new {
     # gives a quick optimisation around testing things later on
 
     if( defined $ann || defined $pid || defined $feat || defined $species ) {
-	$pid && $self->primary_id($pid);
-	$species && $self->species($species);
-	$ann && $self->annotation($ann);
-	
-	if( defined $feat ) {
-	    if( ref($feat) !~ /ARRAY/i ) {
-		if( ref($feat) && $feat->isa('Bio::SeqFeatureI') ) {
-		    $self->add_SeqFeature($feat);
-		} else {
-		    $self->warn("Must specify a valid Bio::SeqFeatureI or ArrayRef of Bio::SeqFeatureI's with the -features init parameter for ".ref($self));
-		}
-	    } else {
-		foreach my $feature ( @$feat ) {
-		    $self->add_SeqFeature($feature);
-		}	
-	    }
-	}
+        $pid && $self->primary_id($pid);
+        $species && $self->species($species);
+        $ann && $self->annotation($ann);
+        
+        if( defined $feat ) {
+            if( ref($feat) !~ /ARRAY/i ) {
+                if( ref($feat) && $feat->isa('Bio::SeqFeatureI') ) {
+                    $self->add_SeqFeature($feat);
+                } else {
+                    $self->warn("Must specify a valid Bio::SeqFeatureI or ArrayRef of Bio::SeqFeatureI's with the -features init parameter for ".ref($self));
+                }
+            } else {
+                foreach my $feature ( @$feat ) {
+                    $self->add_SeqFeature($feature);
+                }
+            }
+        }
     }
 
     return $self;
 }
 
+
 =head1 PrimarySeq interface
 
 
@@ -547,7 +560,7 @@ of course, you are free to use these functions anyway.
  Returns : A scalar
  Args    : Optionally on set the new value (a string). An optional second
            argument presets the alphabet (otherwise it will be guessed).
-           Both parameters may also be given in named paramater style
+           Both parameters may also be given in named parameter style
            with -seq and -alphabet being the names.
 
 =cut
@@ -560,22 +573,17 @@ sub seq {
 =head2 validate_seq
 
  Title   : validate_seq
- Usage   : if(! $seq->validate_seq($seq_str) ) {
-                print "sequence $seq_str is not valid for an object of type ",
-		      ref($seq), "\n";
-	   }
- Function: Validates a given sequence string. A validating sequence string
-           must be accepted by seq(). A string that does not validate will
-           lead to an exception if passed to seq().
-
-           The implementation provided here does not take alphabet() into
-           account. Allowed are all letters (A-Z), '-','.','*','=', and '~'.
-
- Example :
- Returns : 1 if the supplied sequence string is valid for the object, and
-           0 otherwise.
- Args    : The sequence string to be validated.
-
+ Usage   : if(! $seqobj->validate_seq($seq_str) ) {
+                print "sequence $seq_str is not valid for an object of
+                alphabet ",$seqobj->alphabet, "\n";
+           }
+ Function: Test that the given sequence is valid, i.e. contains only valid
+           characters. The allowed characters are all letters (A-Z) and '-','.',
+           '*','?','=' and '~'. Spaces are not valid. Note that this
+           implementation does not take alphabet() into account.
+ Returns : 1 if the supplied sequence string is valid, 0 otherwise.
+ Args    : - Sequence string to be validated
+           - Boolean to throw an error if the sequence is invalid
 
 =cut
 
@@ -583,6 +591,7 @@ sub validate_seq {
     return shift->primary_seq()->validate_seq(@_);
 }
 
+
 =head2 length
 
  Title   : length
@@ -598,9 +607,8 @@ sub length {
     return shift->primary_seq()->length(@_);
 }
 
-=head1 Methods from the Bio::PrimarySeqI interface
 
-=cut
+=head1 Methods from the Bio::PrimarySeqI interface
 
 =head2 subseq
 
@@ -622,6 +630,7 @@ sub subseq {
     return shift->primary_seq()->subseq(@_);
 }
 
+
 =head2 display_id
 
  Title   : display_id
@@ -645,7 +654,6 @@ sub subseq {
  Returns : A string
  Args    : None or a new id
 
-
 =cut
 
 sub display_id {
@@ -653,7 +661,6 @@ sub display_id {
 }
 
 
-
 =head2 accession_number
 
  Title   : accession_number
@@ -673,13 +680,13 @@ sub display_id {
  Returns : A string
  Args    : None or an accession number
 
-
 =cut
 
 sub accession_number {
    return shift->primary_seq->accession_number(@_);
 }
 
+
 =head2 desc
 
  Title   : desc
@@ -689,13 +696,13 @@ sub accession_number {
  Returns : The description
  Args    : The description or none
 
-
 =cut
 
 sub desc {
    return shift->primary_seq->desc(@_);
 }
 
+
 =head2 primary_id
 
  Title   : primary_id
@@ -716,7 +723,6 @@ sub desc {
  Returns : A string
  Args    : None or an id, or undef to unset the primary id.
 
-
 =cut
 
 sub primary_id {
@@ -735,12 +741,13 @@ sub primary_id {
     return shift->primary_seq->primary_id(@_);
 }
 
+
 =head2 can_call_new
 
  Title   : can_call_new
  Usage   : if ( $obj->can_call_new ) {
-             $newobj = $obj->new( %param );
-	   }
+               $newobj = $obj->new( %param );
+           }
  Function: can_call_new returns 1 or 0 depending
            on whether an implementation allows new
            constructor to be called. If a new constructor
@@ -748,21 +755,21 @@ sub primary_id {
            constructor list.
 
            $myobject->new( -seq => $sequence_as_string,
-			   -display_id  => $id
-			   -accession_number => $accession
-			   -alphabet => 'dna',
-			 );
+                           -display_id  => $id
+                           -accession_number => $accession
+                           -alphabet => 'dna',
+                         );
  Example :
  Returns : 1 or 0
  Args    : None
 
-
 =cut
 
 sub can_call_new {
     return 1;
 }
 
+
 =head2 alphabet
 
  Title   : alphabet
@@ -778,7 +785,6 @@ sub can_call_new {
            has to guess.
  Args    : optional string to set : 'dna' | 'rna' | 'protein'
 
-
 =cut
 
 sub alphabet {
@@ -787,6 +793,7 @@ sub alphabet {
    return $self->primary_seq->alphabet();
 }
 
+
 =head2 is_circular
 
  Title   : is_circular
@@ -804,8 +811,6 @@ sub is_circular {
 
 =head1 Methods for Bio::IdentifiableI compliance
 
-=cut
-
 =head2 object_id
 
  Title   : object_id
@@ -817,13 +822,13 @@ sub is_circular {
            This is aliased to accession_number().
  Returns : A scalar
 
-
 =cut
 
 sub object_id {
     return shift->accession_number(@_);
 }
 
+
 =head2 version
 
  Title   : version
@@ -858,6 +863,7 @@ sub authority {
     return shift->primary_seq()->authority(@_);
 }
 
+
 =head2 namespace
 
  Title   : namespace
@@ -868,16 +874,14 @@ sub authority {
 
  Returns : A scalar
 
-
 =cut
 
 sub namespace{
     return shift->primary_seq()->namespace(@_);
 }
 
-=head1 Methods for Bio::DescribableI compliance
 
-=cut
+=head1 Methods for Bio::DescribableI compliance
 
 =head2 display_name
 
@@ -918,9 +922,8 @@ sub description {
     return shift->desc(@_);
 }
 
-=head1 Methods for implementing Bio::AnnotatableI
 
-=cut
+=head1 Methods for implementing Bio::AnnotatableI
 
 =head2 annotation
 
@@ -939,16 +942,89 @@ for more information
 sub annotation {
     my ($obj,$value) = @_;
     if( defined $value ) {
-	$obj->throw("object of class ".ref($value)." does not implement ".
-		    "Bio::AnnotationCollectionI. Too bad.")
-	    unless $value->isa("Bio::AnnotationCollectionI");
-	$obj->{'_annotation'} = $value;
+        $obj->throw("object of class ".ref($value)." does not implement ".
+                    "Bio::AnnotationCollectionI. Too bad.")
+            unless $value->isa("Bio::AnnotationCollectionI");
+        $obj->{'_annotation'} = $value;
     } elsif( ! defined $obj->{'_annotation'}) {
-	$obj->{'_annotation'} = Bio::Annotation::Collection->new();
+        $obj->{'_annotation'} = Bio::Annotation::Collection->new();
     }
     return $obj->{'_annotation'};
 }
 
+
+=head1 Methods for delegating Bio::AnnotationCollectionI
+
+=head2 get_Annotations()
+
+ Usage   : my @annotations = $seq->get_Annotations('key')
+ Function: Retrieves all the Bio::AnnotationI objects for a specific key
+           for this object
+ Returns : list of Bio::AnnotationI - empty if no objects stored for a key
+ Args    : string which is key for annotations
+
+=cut
+
+sub get_Annotations { shift->annotation->get_Annotations(@_); }
+
+
+=head2 add_Annotation()
+
+ Usage   : $seq->add_Annotation('reference',$object);
+           $seq->add_Annotation($object,'Bio::MyInterface::DiseaseI');
+           $seq->add_Annotation($object);
+           $seq->add_Annotation('disease',$object,'Bio::MyInterface::DiseaseI');
+ Function: Adds an annotation for a specific key for this sequence object.
+
+           If the key is omitted, the object to be added must provide a value
+           via its tagname().
+
+           If the archetype is provided, this and future objects added under
+           that tag have to comply with the archetype and will be rejected
+           otherwise.
+
+ Returns : none
+ Args    : annotation key ('disease', 'dblink', ...)
+           object to store (must be Bio::AnnotationI compliant)
+           [optional] object archetype to map future storage of object
+           of these types to
+
+=cut
+
+sub add_Annotation { shift->annotation->add_Annotation(@_) }
+
+
+=head2 remove_Annotations()
+
+ Usage   : $seq->remove_Annotations()
+ Function: Remove the annotations for the specified key from this sequence 
+           object
+ Returns : an list of Bio::AnnotationI compliant objects which were stored
+           under the given key(s) for this sequence object
+ Args    : the key(s) (tag name(s), one or more strings) for which to
+           remove annotations (optional; if none given, flushes all
+           annotations)
+
+=cut
+
+sub remove_Annotations { shift->annotation->remove_Annotations(@_) }
+
+
+=head2 get_num_of_annotations()
+
+ Usage   : my $count = $seq->get_num_of_annotations()
+ Alias   : num_Annotations
+ Function: Returns the count of all annotations stored for this sequence
+           object      
+ Returns : integer
+ Args    : none
+
+=cut
+
+sub get_num_of_annotations { shift->annotation->get_num_of_annotations(@_) }
+sub num_Annotations { shift->get_num_of_annotations }; #DWYM
+
+
 =head1 Methods to implement Bio::FeatureHolderI
 
 This includes methods for retrieving, adding, and removing features.
@@ -967,28 +1043,34 @@ This includes methods for retrieving, adding, and removing features.
            feature object in order to traverse all features associated
            with this sequence.
 
+           Top-level features can be obtained by tag, specified in 
+           the argument.
+
            Use get_all_SeqFeatures() if you want the feature tree
            flattened into one single array.
 
  Example :
  Returns : an array of Bio::SeqFeatureI implementing objects
- Args    : none
-
-At some day we may want to expand this method to allow for a feature
-filter to be passed in.
+ Args    : [optional] scalar string (feature tag)
 
 =cut
 
 sub get_SeqFeatures{
    my $self = shift;
+   my $tag = shift;
 
    if( !defined $self->{'_as_feat'} ) {
        $self->{'_as_feat'} = [];
    }
-
-   return @{$self->{'_as_feat'}};
+   if ($tag) {
+       return map { $_->primary_tag eq $tag ? $_ : () } @{$self->{'_as_feat'}};
+   }
+   else {
+       return @{$self->{'_as_feat'}};
+   }
 }
 
+
 =head2 get_all_SeqFeatures
 
  Title   : get_all_SeqFeatures
@@ -1005,7 +1087,6 @@ sub get_SeqFeatures{
  Returns : An array of Bio::SeqFeatureI implementing objects.
  Args    : None
 
-
 =cut
 
 # this implementation is inherited from FeatureHolderI
@@ -1018,42 +1099,54 @@ sub get_SeqFeatures{
  Returns : integer representing the number of SeqFeatures
  Args    : None
 
-
 =cut
 
 sub feature_count {
     my ($self) = @_;
 
     if (defined($self->{'_as_feat'})) {
-	return ($#{$self->{'_as_feat'}} + 1);
+        return ($#{$self->{'_as_feat'}} + 1);
     } else {
-	return 0;
+        return 0;
     }
 }
 
+
 =head2 add_SeqFeature
 
  Title   : add_SeqFeature
  Usage   : $seq->add_SeqFeature($feat);
-           $seq->add_SeqFeature(@feat);
- Function: Adds the given feature object (or each of an array of feature
-           objects to the feature array of this
+ Function: Adds the given feature object to the feature array of this
            sequence. The object passed is required to implement the
            Bio::SeqFeatureI interface.
+           The 'EXPAND' qualifier (see L<Bio::FeatureHolderI>) is supported, but
+           has no effect,
  Returns : 1 on success
- Args    : A Bio::SeqFeatureI implementing object, or an array of such objects.
-
+ Args    : A Bio::SeqFeatureI implementing object.
 
 =cut
 
 sub add_SeqFeature {
-   my ($self,@feat) = @_;
+   my ($self, @feat) = @_;
 
    $self->{'_as_feat'} = [] unless $self->{'_as_feat'};
 
-   foreach my $feat ( @feat ) {
+   if (scalar @feat > 1) {
+      $self->deprecated(
+         -message => 'Providing an array of features to Bio::Seq add_SeqFeature()'.
+                     ' is deprecated and will be removed in a future version. '.
+                     'Add a single feature at a time instead.',
+         -warn_version    => 1.007,
+         -throw_version   => 1.009,
+      );
+   }
+
+   for my $feat ( @feat ) {
+
+       next if $feat eq 'EXPAND'; # Need to support it for FeatureHolderI compliance
+
        if( !$feat->isa("Bio::SeqFeatureI") ) {
-	   $self->throw("$feat is not a SeqFeatureI and that's what we expect...");
+           $self->throw("Expected a Bio::SeqFeatureI object, but got a $feat.");
        }
 
        # make sure we attach ourselves to the feature if the feature wants it
@@ -1065,6 +1158,7 @@ sub add_SeqFeature {
    return 1;
 }
 
+
 =head2 remove_SeqFeatures
 
  Title   : remove_SeqFeatures
@@ -1077,7 +1171,6 @@ sub add_SeqFeature {
  Returns : The array of Bio::SeqFeatureI objects removed from this seq.
  Args    : None
 
-
 =cut
 
 sub remove_SeqFeatures {
@@ -1089,15 +1182,15 @@ sub remove_SeqFeatures {
     return @feats;
 }
 
-=head1 Methods provided in the Bio::PrimarySeqI interface
 
+=head1 Methods provided in the Bio::PrimarySeqI interface
 
 These methods are inherited from the PrimarySeq interface
 and work as one expects, building new Bio::Seq objects
 or other information as expected. See L<Bio::PrimarySeq>
 for more information.
 
-Sequence Features are B<not> transfered to the new objects.
+Sequence Features are B<not> transferred to the new objects.
 This is possibly a mistake. Anyone who feels the urge in
 dealing with this is welcome to give it a go.
 
@@ -1126,9 +1219,6 @@ dealing with this is welcome to give it a go.
  Returns : A new (fresh) Bio::Seq object
  Args    : None
 
-
-=cut
-
 =head2 trunc
 
  Title   : trunc
@@ -1139,9 +1229,6 @@ dealing with this is welcome to give it a go.
  Returns : A fresh Seq object
  Args    : A Seq object
 
-
-=cut
-
 =head2 id
 
  Title   : id
@@ -1150,7 +1237,6 @@ dealing with this is welcome to give it a go.
  Returns : value of display_id()
  Args    : [optional] value to update display_id
 
-
 =cut
 
 sub  id {
@@ -1160,7 +1246,6 @@ sub  id {
 
 =head1 Seq only methods
 
-
 These methods are specific to the Bio::Seq object, and not
 found on the Bio::PrimarySeq object
 
@@ -1173,7 +1258,6 @@ found on the Bio::PrimarySeq object
  Returns : PrimarySeq object
  Args    : None or PrimarySeq object
 
-
 =cut
 
 sub primary_seq {
@@ -1181,7 +1265,7 @@ sub primary_seq {
 
    if( defined $value) {
        if( ! ref $value || ! $value->isa('Bio::PrimarySeqI') ) {
-	   $obj->throw("$value is not a Bio::PrimarySeq compliant object");
+           $obj->throw("$value is not a Bio::PrimarySeq compliant object");
        }
 
        $obj->{'primary_seq'} = $value;
@@ -1189,7 +1273,7 @@ sub primary_seq {
        # want an attached seq.
 
        foreach my $sf ( $obj->get_SeqFeatures() ) {
-	   $sf->attach_seq($value);
+           $sf->attach_seq($value);
        }
 
    }
@@ -1197,6 +1281,7 @@ sub primary_seq {
 
 }
 
+
 =head2 species
 
  Title   : species
@@ -1218,9 +1303,8 @@ sub species {
     }
 }
 
-=head1 Internal methods
 
-=cut
+# Internal methods follow...
 
 # keep AUTOLOAD happy
 sub DESTROY { }
@@ -1245,7 +1329,7 @@ sub all_SeqFeatures{
 sub accession {
     my $self = shift;
     $self->warn(ref($self)."::accession is deprecated, ".
-		"use accession_number() instead");
+                "use accession_number() instead");
     return $self->accession_number(@_);
 }
 
