@@ -7,9 +7,10 @@
 # POD documentation - main docs before the code
 #
 # The original version of the module can be found here:
-# http://www.lifecde.com/products/agave/agave.pm
+# http://www.agavexml.org/
 #
-# The DTD for AGAVE XML can be located here:
+# ### TODO: live link for this anymore?
+# The DTD for AGAVE XML was once located here (dead link):
 # http://www.lifecde.com/products/agave/schema/v2_3/agave.dtd
 #
 #
@@ -56,13 +57,24 @@ of the Bioperl mailing lists.  Your participation is much appreciated.
   bioperl-l@bioperl.org                  - General discussion
   http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
 
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
+
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution.
 Bug reports can be submitted via the web:
 
-  http://bugzilla.open-bio.org/
+  https://redmine.open-bio.org/projects/bioperl/
 
 =head1 AUTHOR - Simon K. Chan
 
@@ -1505,8 +1517,8 @@ sub  _write_each_record {
     my ($self,$seq) = @_;
 
     # $self->{'file'} =~ s/>//g;
-    my $output = new IO::File(">" . $self->{'file'});
-    my $writer = new XML::Writer(OUTPUT => $output,
+    my $output = IO::File->new(">" . $self->{'file'});
+    my $writer = XML::Writer->new(OUTPUT => $output,
                                  NAMESPACES => 0,
                                  DATA_MODE => 1,
                                  DATA_INDENT => 2 ) ;
