@@ -1,7 +1,8 @@
-# $Id: gameWriter.pm 10525 2006-09-26 22:03:22Z sendu $
 #
 # BioPerl module for Bio::SeqIO::game::gameWriter
 #
+# Please direct questions and support issues to <bioperl-l@bioperl.org> 
+#
 # Cared for by Sheldon McKay <mckays@cshl.edu>
 #
 # You may distribute this module under the same terms as perl itself
@@ -47,13 +48,24 @@ Your participation is much appreciated.
   bioperl-l@bioperl.org                  - General discussion
   http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
 
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
+
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
 of the bugs and their resolution. Bug reports can be submitted via the
 web:
 
-  http://bugzilla.open-bio.org/
+  https://redmine.open-bio.org/projects/bioperl/
 
 =head1 AUTHOR - Sheldon McKay
 
@@ -165,7 +177,7 @@ sub write_to_game {
     # $self->_rearrange_hierarchies($seq, @gene_containers);
 
     # add back nested feats
-    $seq->add_SeqFeature( @nested_feats  );
+    $seq->add_SeqFeature( $_ ) foreach @nested_feats;
     
     my $atts  = {};
     my $xml = '';
@@ -283,7 +295,7 @@ sub _rearrange_hierarchies { #renamed to not conflict with Bio::Root::_rearrange
     }    
    
     push @addback, (@containers, grep { defined $_ } @genes );
-    $seq->add_SeqFeature(@addback);
+    $seq->add_SeqFeature($_) foreach @addback;
 }
 
 
