@@ -1,4 +1,3 @@
-# $Id: StandAloneNCBIBlast.pm 15383 2009-01-16 16:45:08Z cjfields $
 #
 # BioPerl module for Bio::Tools::Run::StandAloneBlast
 #
@@ -33,13 +32,24 @@ of the Bioperl mailing lists.  Your participation is much appreciated.
   bioperl-l@bioperl.org                  - General discussion
   http://bioperl.org/wiki/Mailing_lists  - About the mailing lists
 
+=head2 Support 
+
+Please direct usage questions or support issues to the mailing list:
+
+I<bioperl-l@bioperl.org>
+
+rather than to the module maintainer directly. Many experienced and 
+reponsive experts will be able look at the problem and quickly 
+address it. Please include a thorough description of the problem 
+with code and data examples if at all possible.
+
 =head2 Reporting Bugs
 
 Report bugs to the Bioperl bug tracking system to help us keep track
 the bugs and their resolution.  Bug reports can be submitted via 
 the web:
 
-  http://bugzilla.open-bio.org/
+  https://redmine.open-bio.org/projects/bioperl/
 
 =head1 AUTHOR - Peter Schattner
 
@@ -63,6 +73,7 @@ methods. Internal methods are usually preceded with a _
 package Bio::Tools::Run::StandAloneNCBIBlast;
 
 use strict;
+use warnings;
 
 use base qw(Bio::Tools::Run::StandAloneBlast);
 
@@ -78,7 +89,7 @@ our %GENERAL_PARAMS  = (i => 'input',
                         o => 'outfile',
                         p => 'program',
                         d => 'database');
-our @BLASTALL_PARAMS = qw(A B C D E F G K L M O P Q R S W X Y Z a b e f l m q r t v w y z);
+our @BLASTALL_PARAMS = qw(A B C D E F G K L M O P Q R S W X Y Z a b e f l m q r t v w y z n);
 our @BLASTALL_SWITCH = qw(I g J T U n V s);
 our @BLASTPGP_PARAMS = qw(A B C E F G H I J K L M N O P Q R S T U W X Y Z a b c e f h j k l m q s t u v y z);
 our @RPSBLAST_PARAMS = qw(F I J L N O P T U V X Y Z a b e l m v y z);
@@ -102,7 +113,7 @@ Some of the most commonly used parameters are listed below. All
 parameters have defaults and are optional except for -p in those programs that
 have it. For a complete listing of settable parameters, run the relevant
 executable BLAST program with the option "-" as in blastall -
-Note that the input paramters (-i, -j, -input) should not be set directly by
+Note that the input parameters (-i, -j, -input) should not be set directly by
 you: this module sets them when you call one of the executable methods.
 
 Blastall
